在建模过程中，建筑师和工程师并不需要对所有构件进行三维建模，也可通过创建标准详图，以说明如何构造较大项目中的材质。详图是对项目的重要补充，因为它们显示了材质应该如何相互连接。<br />
本节将介绍在详图中使用的以及可以通过BIMBase API访问的详图线对象。详图线对象包括直线、圆、圆弧、多段线四类，如表7-1所示。<br />
<div style="text-align: center;">
  <p>表7-1 详图线对象</p>
</div>

|直线对象|BPModelLine|
|-----|-----|
|圆对象|BPModelCircle|
|圆弧对象|BPModelArc3d|
|多段线对象|BPPolyLineEntity|

可通过调用相应接口创建详图线对象，示例代码7-1中通过构造函数创建了线（`BPModelLine`）对象，并将线添加到活动模型空间上。<br />

```cpp
BPProjectP pProject = BPApplication::getInstance().getProjectManager()->getMainProject();
if (pProject == nullptr)
	return;
BPModelBaseP pModel = pProject->getActiveModel();
if (pModel == nullptr)
	return;
//创建线对象
BPModelLine line(GePoint3d::create(3000, 3000, 0), GePoint3d::create(4000, 4000, 0));
//将对象保存到model
::p3d::P3DStatus status = line.addToProject(*pProject, pModel->getModelId());

```
<div style="text-align: center;">
  <p>代码7-1：详图线对象的创建</p>
</div>

<div style="text-align: center;">
  <p>表7-2 详图线对象接口</p>
</div>

<style>
    table {
        text-align: center;
    }
</style>

<table>
    <tr>
        <td rowspan="7">直线对象BPModelLine
</td>
        <td>Create(GePoint3dCR,GePoint3dCR，GeVec3dCR )</td>
		<td>创建直线对象</td>
    </tr>
    <tr>
        <td>GetStartPoint()const</td>
		<td>获得直线起始点</td>
    </tr>
	<tr>
        <td>SetStartPoint(GePoint3dCR)</td>
		<td>设置直线起始点</td>
    </tr>
	    <tr>
        <td>GetMidPoint()const</td>
        <td>获得直线中点</td>
    </tr>
	    <tr>
        <td>GetEndPoint()const</td>
		<td>获得直线末端点</td>
	</tr>
	<tr>
	<td>SetEndPoint(GePoint3dCR)</td>
		<td>设置直线末端点</td>
	</tr>
	<tr>
	<td>GetDirection()</td>
		<td>获得直线方向向量</td>
	</tr>
	<tr>
        <td rowspan="6">圆对象BPModelCircle
</td>
        <td>Create(GePoint3dCR center,GeVec3dCR normal, double radius)</td>
		<td>创建圆对象（圆心，方向向量，半径）</td>
    </tr>
    <tr>
        <td>GePoint3d GetCenter()const</td>
		<td>获得圆心点</td>
    </tr>
	<tr>
        <td>SetCenter(GePoint3dCR center)</td>
		<td>设置圆心点</td>
    </tr>
	    <tr>
        <td>GeVec3d GetNormal()const</td>
        <td>方向向量</td>
    </tr>
	    <tr>
        <td>double GetLength()const</td>
		<td>获得圆周长</td>
	</tr>
	<tr>
	<td>double GetArea()const</td>
		<td>获得圆面积</td>
	</tr>
	<tr>
        <td rowspan="9">圆弧对象BPModelArc3d
</td>
        <td>Create(GePoint3dCR startPoint, GePoint3dCR middlePoint, GePoint3dCR endPoint)</td>
		<td>创建圆弧对象（三点确定圆弧）</td>
    </tr>
    <tr>
        <td>GePoint3d GetStartPoint()const</td>
		<td>获得圆弧起始点</td>
    </tr>
	<tr>
        <td>GePoint3d GetEndPoint()const</td>
		<td>获得圆弧终止点</td>
    </tr>
	    <tr>
        <td>GePoint3d GetMidPoint()const</td>
        <td>获得圆弧中间点</td>
    </tr>
	    <tr>
        <td>GePoint3d GetCenter()const</td>
		<td>获得圆弧圆心点</td>
	</tr>
	<tr>
	<td>double Length()const</td>
		<td>获得圆弧长度</td>
	</tr>
	<tr>
        <td>GeVec3d GetXDir()const</td>
        <td>X方向向量</td>
    </tr>
	    <tr>
        <td>GeVec3d GetYDir()const</td>
		<td>Y方向向量</td>
	</tr>
	<tr>
	<td>GeVec3d GetNormal()const</td>
		<td>方向向量</td>
	</tr>
	<tr>
        <td rowspan="9">多段线对象BPPolyLineEntity
</td>
        <td>addVertex(GePoint3d&, GePoint3d&， bool isArc, double startWidth, double endWidth)</td>
		<td>	添加点</td>
    </tr>
    <tr>
        <td>int numVerts()</td>
		<td>获取点的数量</td>
    </tr>
	<tr>
        <td>getVertex(int index, GePoint3d& pt, GePoint3d& ptOhter, bool& isArc, double& startWidth, double& endWidth)</td>
		<td>获取点的信息</td>
    </tr>
	    <tr>
        <td>clearVertexs()</td>
        <td>清空所有点的信息</td>
    </tr>
	    <tr>
        <td>	setClosed(bool closed)</td>
		<td>设置是否闭合</td>
	</tr>
	<tr>
	<td>bool closed()</td>
		<td>获得是否闭合</td>
	</tr>
	<tr>
        <td>setSegments(list& segments)</td>
        <td>	设置多段线list</td>
    </tr>
	    <tr>
        <td>getSegments(list& segments)</td>
		<td>	获得多段线list</td>
	</tr>
	<tr>
	<td>CreatePolylineCurveVec(bool isClosed, const list&segments,vector& curveArrays)</td>
		<td>	创建多段线的GeCurveArray</td>
	</tr>